# This file was created by fish when upgrading to version 4.3, to migrate
# theme variables from universal to global scope.
# Don't edit this file, as it will be written by the web-config tool (`fish_config`).
# To customize your theme, delete this file and see
#     help interactive#syntax-highlighting
# or
#     man fish-interactive | less +/^SYNTAX.HIGHLIGHTING
# for appropriate commands to add to ~/.config/fish/config.fish instead.
# See also the release notes for fish 4.3.0 (run `help relnotes`).

set --global fish_color_autosuggestion 767676
set --global fish_color_cancel --reverse
set --global fish_color_command 87d787
set --global fish_color_comment 6c6c6c
set --global fish_color_cwd green
set --global fish_color_cwd_root red
set --global fish_color_end ff8787
set --global fish_color_error cc6666
set --global fish_color_escape 00a6b2
set --global fish_color_history_current --bold
set --global fish_color_host normal
set --global fish_color_host_remote
set --global fish_color_keyword
set --global fish_color_match --background=brblue
set --global fish_color_normal normal
set --global fish_color_operator 00a6b2
set --global fish_color_option
set --global fish_color_param ffd787
set --global fish_color_quote 87d7d7
set --global fish_color_redirection 00afd7
set --global fish_color_search_match white --background=brblack
set --global fish_color_selection white --bold --background=brblack
set --global fish_color_status red
set --global fish_color_user brgreen
set --global fish_color_valid_path --underline
set --global fish_pager_color_background
set --global fish_pager_color_completion normal
set --global fish_pager_color_description B3A06D
set --global fish_pager_color_prefix normal --bold --underline
set --global fish_pager_color_progress brwhite --background=cyan
set --global fish_pager_color_secondary_background
set --global fish_pager_color_secondary_completion
set --global fish_pager_color_secondary_description
set --global fish_pager_color_secondary_prefix
set --global fish_pager_color_selected_background --background=brblack
set --global fish_pager_color_selected_completion
set --global fish_pager_color_selected_description
set --global fish_pager_color_selected_prefix
